@@ -51,6 +51,9 @@ pub fn analyse_multithreaded(
     graph.text(&format!("Illuminant: D(T={:.2}°K)", T),
                w - 2, 9, TextAnchor::ne(), font.as_ref(),
                palette.tl_rgb);
+    graph.text(metadata::REPO,
+               w - 3, h - 2, TextAnchor::se(), font.as_ref(),
+               palette.tl_rgb);
 
     fn init_state<'a, T: GraphPixel>(
                 graph: &'a mut ImageGraph,
@@ -174,6 +177,9 @@ pub fn analyse_singlethreaded<CP: CacheProvider, C: AsRef<RwLock<CP>>+Clone, FR:
     graph.text(&format!("Illuminant: D(T={:.2}°K)", T),
                w - 2, 9, TextAnchor::ne(), font.as_ref(),
                palette.tl_rgb);
+    graph.text(metadata::REPO,
+               w - 3, h - 2, TextAnchor::se(), font.as_ref(),
+               palette.tl_rgb);
 
     let graph_rw = Rc::new(RwLock::new(graph));
     analyse_main(
